// --------------------------------------------------------------------------------------------------------------------
// <copyright file="DropDownListProjection.cs" company="Devbridge Group LLC">
// 
// Copyright (C) 2015,2016 Devbridge Group LLC
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU Lesser General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU Lesser General Public License
// along with this program.  If not, see http://www.gnu.org/licenses/. 
// </copyright>
// 
// <summary>
// Better CMS is a publishing focused and developer friendly .NET open source CMS.
// 
// Website: https://www.bettercms.com 
// GitHub: https://github.com/devbridge/bettercms
// Email: info@bettercms.com
// </summary>
// --------------------------------------------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using System.Web.Mvc;
using System.Web.UI;
using System.Web.UI.HtmlControls;

using BetterCms.Core.DataContracts;

using BetterModules.Core.Web.Modules;

namespace BetterCms.Core.Modules.Projections
{
    /// <summary>
    /// Defines client side action attached to side menu button.
    /// </summary>
    public class DropDownListProjection : HtmlElementProjection
    {
        /// <summary>
        /// Module action marker css class.
        /// </summary>
        private const string ModuleActionMarkerCssClass = "bcms-onchange-action";

        /// <summary>
        /// Html element attribute name to mark module name.
        /// </summary>
        private const string ModuleNameAttribute = "data-bcms-module";

        /// <summary>
        /// Html element attribute name to mark module action name.
        /// </summary>
        private const string ModuleActionAttribute = "data-bcms-action";

        /// <summary>
        /// Gets or sets the client side parent module.
        /// </summary>
        /// <value>
        /// The client side parent module.
        /// </value>
        private readonly JsIncludeDescriptor parentModuleInclude;

        /// <summary>
        /// Initializes a new instance of the <see cref="DropDownListProjection" /> class.
        /// </summary>
        /// <param name="parentModuleInclude">The parent module (should contain on change action).</param>
        /// <param name="items">A list of select items.</param>
        /// <param name="onChangeAction">Name of the action to execute after select item changed.</param>
        [SuppressMessage("StyleCop.CSharp.NamingRules", "SA1305:FieldNamesMustNotUseHungarianNotation", Justification = "Reviewed. Suppression is OK here.")]
        public DropDownListProjection(JsIncludeDescriptor parentModuleInclude, IEnumerable<Func<IPage, DropDownListProjectionItem>> items, Func<IPage, string> onChangeAction)
            : base("div")
        {
            this.parentModuleInclude = parentModuleInclude;
            Items = items;
            OnChangeAction = onChangeAction;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="DropDownListProjection" /> class.
        /// </summary>
        /// <param name="parentModuleInclude">The parent module (should contain on change action).</param>
        /// <param name="onChangeAction">Name of the action to execute after select item changed.</param>
        [SuppressMessage("StyleCop.CSharp.NamingRules", "SA1305:FieldNamesMustNotUseHungarianNotation", Justification = "Reviewed. Suppression is OK here.")]
        public DropDownListProjection(JsIncludeDescriptor parentModuleInclude, Func<IPage, string> onChangeAction)
            : base("div")
        {
            this.parentModuleInclude = parentModuleInclude;
            OnChangeAction = onChangeAction;
        }

        /// <summary>
        /// Gets or sets drop down list items enumerator.
        /// </summary>
        public IEnumerable<Func<IPage, DropDownListProjectionItem>> Items { get; set; }

        /// <summary>
        /// Gets or sets the callback function attached to action.
        /// </summary>
        /// <value>
        /// The callback function.
        /// </value>
        public Func<IPage, string> OnChangeAction { get; set; }

        /// <summary>
        /// Called before render methods sends element to response output.
        /// </summary>
        /// <param name="page">The page.</param>
        /// <param name="html">The html helper.</param>
        protected override void OnPreRender(HtmlControlRenderer controlRenderer, IPage page, HtmlHelper html)
        {
            base.OnPreRender(controlRenderer, page, html);

            if (Items != null && Items.Any())
            {
                var innerDiv = new HtmlGenericControl("div");
                controlRenderer.Controls.Add(innerDiv);
                var items = Items
                    .Select(f => f(page))
                    .OrderBy(f => f.Order)
                    .Select(
                        f => new SelectListItem
                        {
                            Selected = f.IsSelected,
                            Text = f.Text != null ? f.Text() : string.Empty,
                            Value = f.Value
                        })
                    .ToList();

                foreach (var item in items)
                {
                    var option = new HtmlGenericControl("div");
                    option.Controls.Add(new LiteralControl(item.Text));
                    option.Attributes["value"] = item.Value;
                    if (item.Selected)
                    {
                        option.Attributes["selected"] = "selected";
                    }

                    innerDiv.Controls.Add(option);
                }
            }

            if (OnChangeAction != null && parentModuleInclude != null)
            {
                string cssClass = controlRenderer.Attributes["class"];
                if (!string.IsNullOrEmpty(cssClass) && !cssClass.Contains(ModuleActionMarkerCssClass))
                {
                    cssClass = string.Concat(cssClass, " ", ModuleActionMarkerCssClass);
                }
                else
                {
                    cssClass = ModuleActionMarkerCssClass;
                }

                controlRenderer.Attributes["class"] = cssClass;
                controlRenderer.Attributes.Add(ModuleNameAttribute, parentModuleInclude.Name);
                controlRenderer.Attributes.Add(ModuleActionAttribute, OnChangeAction(page));
            }
        }
    }
}
